// Generated by Damien version 1.0 alpha 4

package org.eclipse.fdt.internal.core.f95parser;

import java.util.ArrayList;

/**
 * An LALR(1) parser for Fortran 95
 */
public class Parser
{
    /**
     * The lexical analyzer
     */
    private ILexer lexer;

    /**
     * This becomes set to true when we finish parsing
     */
    private boolean doneParsing;

    /**
     * The current state of the parser
     */
    private int currentState;

    /**
     * The next token in the input stream
     */
    private Token lookaheadToken;

    /**
     * Although textbook descriptions of LR parsers suggest a stack containing
     * both states and symbols (terminals and nonterminals), we maintain three
     * parallel stacks: one for states, one for symbols, and one for values
     * returned by user code.  This one holds symbols (terminals and nonterminals).
     */
    private ArrayList symbolStack = new ArrayList();

    /**
     * Although textbook descriptions of LR parsers suggest a stack containing
     * both states and symbols (terminals and nonterminals), we maintain three
     * parallel stacks: one for states, one for symbols, and one for values
     * returned by user code.  This one holds states.
     */
    private ArrayList stateStack = new ArrayList();

    /**
     * Although textbook descriptions of LR parsers suggest a stack containing
     * both states and symbols (terminals and nonterminals), we maintain three
     * parallel stacks: one for states, one for symbols, and one for values
     * returned by user code.  This one holds these values.  When a reduction
     * is performed and user code like <code>return $1 + $2</code> is executed, this
     * is where the values of $1 and $2 come from.
     */
    private ArrayList valueStack = new ArrayList();

    /**
     * The userActions field refers to an instance of the ParserUserActions
     * class, which contains all of the user code
     */
    private ParserUserActions userActions = new ParserUserActions(this);

    /**
     * The entrypoint to the parser.
     */
    public ParseTreeNode parse(ILexer lexicalAnalyzer) throws Exception
    {
        lexer = lexicalAnalyzer;

        // Run any user-specified initialization code
        userActions.initialize();

        // The parser starts in state 0
        currentState = 0;
        stateStack.add(new Integer(currentState));
        lookaheadToken = lexer.yylex();
        doneParsing = false;

        // Repeatedly determine the next action based on the current state
        //System.out.println("Begin parsing...");
        while (!doneParsing)
            ParsingTable.getInstance().processLookaheadFor(this);
        //System.out.println("Done parsing...");

        // Run any user-specified deinitialization code
        userActions.deinitialize();

        // Return the value returned by the last piece of user code
        // executed in a completed parse, the value associated with the
        // start symbol
        return valueStack.isEmpty() ? null : (ParseTreeNode)valueStack.get(valueStack.size()-1);
    }

    /**
     * Shift the next input symbol and change the parser to the given state.
     */
    void shiftAndGoToState(int state) throws Exception
    {
        symbolStack.add(lookaheadToken.getTerminal());
        stateStack.add(new Integer(state));
        valueStack.add(lookaheadToken);
        currentState = state;
        lookaheadToken = lexer.yylex();
    }

    /**
     * Reduce the top symbolsToPop symbols on the stack to the given nonterminal,
     * and change the parser state accordingly.  This overload is called when reducing to an
     * ordinary nonterminal.
     */
    void reduce(Nonterminal nonterminalToReduceTo, int symbolsToPop)
    {
        for (int i = 0; i < symbolsToPop; i++)
        {
            symbolStack.remove(symbolStack.size() - 1);
            stateStack.remove(stateStack.size() - 1);
        }
        currentState = ((Integer)stateStack.get(stateStack.size()-1)).intValue();
        symbolStack.add(nonterminalToReduceTo);
        stateStack.add(new Integer(ParsingTable.getInstance().getGoToFor(nonterminalToReduceTo, this)));
        currentState = ((Integer)stateStack.get(stateStack.size()-1)).intValue();
    }

    /**
     * Reduce the top symbolsToPop symbols on the stack to the given nonterminal,
     * and change the parser state accordingly.  This overload is called when reducing to a
     * nonterminal derived from an EBNF expression: these nonterminals push their own values
     * onto the valueStack, and so the reduce method does not need to.
     */
    void reduce(Nonterminal nonterminalToReduceTo, int symbolsToPop, Object nonterminalValue)
    {
        for (int i = 0; i < symbolsToPop; i++)
        {
            symbolStack.remove(symbolStack.size() - 1);
            stateStack.remove(stateStack.size() - 1);
        }
        currentState = ((Integer)stateStack.get(stateStack.size()-1)).intValue();
        symbolStack.add(nonterminalToReduceTo);
        valueStack.add(nonterminalValue);
        stateStack.add(new Integer(ParsingTable.getInstance().getGoToFor(nonterminalToReduceTo, this)));
        currentState = ((Integer)stateStack.get(stateStack.size()-1)).intValue();
    }

    /**
     * Indicate that parsing is complete
     */
    void accept()
    {
        doneParsing = true;
    }

    /**
     * Stop parsing: A syntax error was found
     */
    void syntaxError() throws Exception
    {
        throw new SyntaxError(lookaheadToken);
    }

    /**
     * Retrieve the parser's current lookahead token
     */
    Token getLookaheadToken() { return lookaheadToken; }

    /**
     * Retrieve the parser's value stack
     */
    ArrayList getValueStack() { return valueStack; }

    /**
     * Retrieve the object containing all of the user code
     */
    ParserUserActions getUserActions() { return userActions; }

    /**
     * Retrieve the parser's current state (an integer)
     */
    int getCurrentState() { return currentState; }
}
